Package: tidypredict
Title: Run Predictions Inside the Database
Version: 1.1.1
Authors@R: c(
    person("Emil", "Hvitfeldt", , "emil.hvitfeldt@posit.co", role = c("aut", "cre")),
    person("Edgar", "Ruiz", , "edgar@posit.co", role = "aut"),
    person("Max", "Kuhn", , "max@posit.co", role = "aut")
  )
Description: It parses a fitted 'R' model object, and returns a formula in
    'Tidy Eval' code that calculates the predictions.  It works with
    several databases back-ends because it leverages 'dplyr' and 'dbplyr'
    for the final 'SQL' translation of the algorithm. It currently
    supports lm(), glm(), randomForest(), ranger(), rpart(), earth(),
    xgb.Booster.complete(), lgb.Booster(), catboost.Model(), cubist(), and
    ctree() models.
License: MIT + file LICENSE
URL: https://tidypredict.tidymodels.org,
        https://github.com/tidymodels/tidypredict
BugReports: https://github.com/tidymodels/tidypredict/issues
Depends: R (>= 3.6)
Imports: cli, dplyr (>= 0.7), generics, jsonlite, knitr, purrr, rlang
        (>= 1.1.1), tibble, tidyr
Suggests: bonsai, covr, Cubist (>= 0.5.1), data.table, DBI, dbplyr,
        earth (>= 5.1.2), glmnet, lightgbm, methods, mlbench,
        modeldata, nycflights13, parsnip, partykit, randomForest,
        ranger (>= 0.14.1), RhpcBLASctl, rpart (>= 4.1.0), rmarkdown,
        RSQLite, survival, testthat (>= 3.2.0), withr, xgboost, yaml
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-23 23:07:51 UTC; emilhvitfeldt
Author: Emil Hvitfeldt [aut, cre],
  Edgar Ruiz [aut],
  Max Kuhn [aut]
Maintainer: Emil Hvitfeldt <emil.hvitfeldt@posit.co>
Repository: CRAN
Date/Publication: 2026-08-24 05:20:02 UTC
