# Generated by roxygen2: do not edit by hand

S3method(acceptable_formula,glm)
S3method(acceptable_formula,lm)
S3method(as_parsed_model,list)
S3method(knit_print,tidypredict_test)
S3method(parse_model,catboost.Model)
S3method(parse_model,cubist)
S3method(parse_model,earth)
S3method(parse_model,glm)
S3method(parse_model,glmnet)
S3method(parse_model,lgb.Booster)
S3method(parse_model,lm)
S3method(parse_model,model_fit)
S3method(parse_model,party)
S3method(parse_model,randomForest)
S3method(parse_model,ranger)
S3method(parse_model,rpart)
S3method(parse_model,xgb.Booster)
S3method(print,tidypredict_test)
S3method(tidy,pm_regression)
S3method(tidypredict_fit,"_xgb.Booster")
S3method(tidypredict_fit,catboost.Model)
S3method(tidypredict_fit,cubist)
S3method(tidypredict_fit,earth)
S3method(tidypredict_fit,glm)
S3method(tidypredict_fit,glmnet)
S3method(tidypredict_fit,lgb.Booster)
S3method(tidypredict_fit,lm)
S3method(tidypredict_fit,model_fit)
S3method(tidypredict_fit,party)
S3method(tidypredict_fit,pm_catboost)
S3method(tidypredict_fit,pm_lgb)
S3method(tidypredict_fit,pm_regression)
S3method(tidypredict_fit,pm_tree)
S3method(tidypredict_fit,pm_xgb)
S3method(tidypredict_fit,randomForest)
S3method(tidypredict_fit,ranger)
S3method(tidypredict_fit,rpart)
S3method(tidypredict_fit,xgb.Booster)
S3method(tidypredict_interval,data.frame)
S3method(tidypredict_interval,glm)
S3method(tidypredict_interval,list)
S3method(tidypredict_interval,lm)
S3method(tidypredict_test,"_xgb.Booster")
S3method(tidypredict_test,catboost.Model)
S3method(tidypredict_test,default)
S3method(tidypredict_test,glmnet)
S3method(tidypredict_test,lgb.Booster)
S3method(tidypredict_test,model_fit)
S3method(tidypredict_test,party)
S3method(tidypredict_test,rpart)
S3method(tidypredict_test,xgb.Booster)
export(.build_case_when_tree)
export(.build_linear_pred)
export(.build_nested_case_when_tree)
export(.extract_catboost_trees)
export(.extract_earth_multiclass)
export(.extract_glmnet_multiclass)
export(.extract_lgb_trees)
export(.extract_partykit_classprob)
export(.extract_ranger_classprob)
export(.extract_ranger_trees)
export(.extract_rf_classprob)
export(.extract_rf_trees)
export(.extract_rpart_classprob)
export(.extract_xgb_trees)
export(.partykit_tree_info_full)
export(.rpart_tree_info_full)
export(acceptable_formula)
export(as_parsed_model)
export(parse_model)
export(set_catboost_categories)
export(tidy)
export(tidypredict_fit)
export(tidypredict_interval)
export(tidypredict_sql)
export(tidypredict_sql_interval)
export(tidypredict_test)
export(tidypredict_to_column)
import(rlang)
importFrom(dplyr,"%>%")
importFrom(dplyr,case_when)
importFrom(dplyr,mutate)
importFrom(generics,tidy)
importFrom(knitr,knit_print)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(purrr,reduce)
importFrom(purrr,transpose)
importFrom(stats,binomial)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,setNames)
importFrom(tibble,tibble)
importFrom(utils,head)
