---
title: "Create a regression spec - version 2"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Create a regression spec - version 2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, echo = FALSE}
spec_frame <- read.csv("regression.csv")
knitr::kable(spec_frame, row.names = FALSE)
```
