# alert triggered with threshold = 0 (mocked)

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 32
      
      Fit max  difference:
      Lower max difference:
      Upper max difference:-<number>

# alert with intervals (mocked)

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 32
      Lower interval records above the threshold: 32
      Upper interval records above the threshold: 32
      
      Fit max  difference:<number>
      Lower max difference:<number>
      Upper max difference:-<number>

# xgboost alert branch

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 18
      
      Max difference: <number>

# glmnet alert branch (mocked)

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 2368
      
      Max difference: -<number>

# lightgbm alert branch (mocked)

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 32
      
      Max difference: <number>

# catboost alert branch (mocked)

    Code
      cat(t$message)
    Output
      tidypredict test results
      Difference threshold: 0
      
      Fitted records above the threshold: 32
      
      Max difference: <number>

