% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree-nested.R
\name{.build_nested_case_when_tree}
\alias{.build_nested_case_when_tree}
\title{Build nested case_when expression from tree info}
\usage{
.build_nested_case_when_tree(tree_info)
}
\arguments{
\item{tree_info}{A tree info list with nodeID, leftChild, rightChild,
splitvarName, terminal, prediction, and node_splits}
}
\description{
Shared helper for building nested tree expressions. This is the nested
equivalent of \code{.build_case_when_tree()}.
}
\keyword{internal}
