% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree.R
\name{.build_case_when_tree}
\alias{.build_case_when_tree}
\title{Build case_when expression from nodes with predictions and paths}
\usage{
.build_case_when_tree(nodes)
}
\arguments{
\item{nodes}{A list of lists, each with \code{prediction} (numeric) and \code{path}}
}
\description{
Shared helper for building tree expressions used by ranger and randomForest
classification extractors.
}
\keyword{internal}
